function [A,B]=CGT(fc,b,phi,order)
% [A,B]=CGT(fc,b,phi,order)
%
% Design a complex-valued gammatone filter.
% Currently, this function supports orders 4 (default) and 6. 
% This function is used by Make_cgt, which is typically used
% through function Make_GTbank.
%
% This file is a part of HUTear- Matlab toolbox for auditory
% modeling. The toolbox is available at 
% http://www.acoustics.hut.fi/software/HUTear/

% Copyrights: Aki Hrm, Helsinki University of Technology, 
% Laboratory of Acoustics and Audio Signal Processing, 
% Espoo, Finland.
% Date: August 20 1999
% Email: Aki.Harma@hut.fi


if nargin==2, phi=0; order=4; end
if nargin==3, order=4; end

switch order, 
case 4, a=1;
	a0 = 0;
	a1 = a*exp(phi-2*pi*b-2*i*pi*fc);
	a2 = 4*a*exp(-2*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);
	a3 = a*exp(-4*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);
	b0 = 1;
	b1 = -4*exp(-2*pi*(b+i*fc));
	b2 = 6*exp(-2*pi*(b+i*fc))^2;
	b3 = -4*exp(-2*pi*(b+i*fc))^3;
	b4 = exp(-2*pi*(b+i*fc))^4;
	A=[a0 a1 a2 a3];B=[b0 b1 b2 b3 b4];
case 6,	a=1;
	a0=0;
	a1 = a*exp(phi-2*pi*b-2*i*pi*fc);
	a2 = 26*a*exp(-2*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);
	a3 = 66*a*exp(-4*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);
	a4 = 26*a*exp(-6*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);
	a5 = a*exp(-8*pi*(b+i*fc))*exp(phi-2*pi*b-2*i*pi*fc);

	b0 = 1;
	b1 = -6*exp(-2*pi*(b+i*fc));
	b2 = 15*exp(-2*pi*(b+i*fc))^2;
	b3 = -20*exp(-2*pi*(b+i*fc))^3;
	b4 = 15*exp(-2*pi*(b+i*fc))^4;
	b5 = -6*exp(-2*pi*(b+i*fc))^5;
	b6 = exp(-2*pi*(b+i*fc))^6;

	A=[a0 a1 a2 a3 a4 a5];B=[b0 b1 b2 b3 b4 b5 b6];
end

ca=polyval(fliplr(A),exp(i*2*pi*fc));
cb=polyval(fliplr(B),exp(i*2*pi*fc));
B=B*ca;A=A*cb;

