function [f,b,CentFreqs]=Make_GTbank;
%    [f,b,CentFreqs]=Make_GTbank;
%
%  Design a customized gammatone filterbank
% 
% There are three designs available: 
% 1 - Conventional real-valued gammatone filterbank based
% on the efficient implementation by Malcolm Slaney
% 2 - A complex-valued implementation of 4th order gammatone
% responses
% 3 - A complex-valued implementation of 6th order gammatone
% functions
%
% This file is a part of HUTear- Matlab toolbox for auditory
% modeling. The toolbox is available at 
% http://www.acoustics.hut.fi/software/HUTear/

% Copyrights: Aki Hrm, Helsinki University of Technology, 
% Laboratory of Acoustics and Audio Signal Processing, 
% Espoo, Finland.
% Date: August 20 1999
% Email: Aki.Harma@hut.fi

disp('************************************************');
disp(' Design a gammatone filterbank ');
disp(' Choose ');
disp(' 1 - Conventional gammatone filterbank');
disp(' 2 - Analytical gammatone filterbank of order 4');
disp(' 3 - Analytical gammatone filterbank of order 6');
sw2=input('Choose:');

switch sw2,
  case 1,
disp('************************************************');
disp(' Make a gammatone filterbank');
disp(' Choose ');
disp(' 1 - to specify all center frequencies in Hz');
disp(' 2 - to specify center frequencies in ERB');
disp(' 3 - give number of channels and specify the lowest ');
disp('     center frequency');

sw1=input('Choose:');
    fs=input('Give sampling rate [Hz]:');    
switch sw1
  case 1,
    cf=input('Give center frequencies [f1 f2 f3 f4 ...][Hz]:');
    [f,b]=MakeERBFiltersB(fs,10,cf);
  case 2,  
    cf=input('Give center frequencies [e1 e2 e3 e4 ...][ERB]:');
    [f,b]=MakeERBFiltersB(fs,10,ERB2Hz(cf));
  case 3,
    NC=input('Give the number of channels:');
    lf=input('Lowest center frequency [Hz]:');
    [f,b]=MakeERBFiltersB(fs,NC,lf);
  otherwise, disp('Ok. Goodbye')
end

  case 2,
disp('****************************************************');
disp(' Make an analytical gammatone filterbank of order 4');
disp(' Choose ');
disp(' 1 - to specify all center frequencies in Hz');
disp(' 2 - to specify center frequencies in ERB');
disp(' 3 - give number of channels and specify the lowest ');
disp('     center frequency');

sw1=input('Choose:');
    fs=input('Give sampling rate [Hz]:');    
switch sw1
  case 1,
    cf=input('Give center frequencies [f1 f2 f3 f4 ...][Hz]:');
    [f,b]=Make_cgtbank(16,fs,cf,4);
  case 2,  
    cf=input('Give center frequencies [e1 e2 e3 e4 ...][ERB]:');
    [f,b]=Make_cgtbank(16,fs,ERB2Hz(cf),4);
  case 3,
    NC=input('Give the number of channels:');
    lf=input('Lowest center frequency [Hz]:');
    [f,b,cf]=Make_cgtbank(NC,fs,lf,4);
  otherwise, disp('Ok. Goodbye')
end

case 3,
disp('****************************************************');
disp(' Make an analytical gammatone filterbank of order 6');
disp(' Choose ');
disp(' 1 - to specify all center frequencies in Hz');
disp(' 2 - to specify center frequencies in ERB');
disp(' 3 - give number of channels and specify the lowest ');
disp('     center frequency');

sw1=input('Choose:');
    fs=input('Give sampling rate [Hz]:');    
switch sw1
  case 1,
    cf=input('Give center frequencies [f1 f2 f3 f4 ...][Hz]:');
    [f,b]=Make_cgtbank(16,fs,cf,6);
  case 2,  
    cf=input('Give center frequencies [e1 e2 e3 e4 ...][ERB]:');
    [f,b]=Make_cgtbank(16,fs,ERB2Hz(cf),6);
    cf=ERB2Hz(cf); % Convert values back to the frequecy scale
  case 3,
    NC=input('Give the number of channels:');
    lf=input('Lowest center frequency [Hz]:');
    [f,b,cf]=Make_cgtbank(NC,fs,lf,6);
  otherwise, disp('Ok. Goodbye')
end
end  

disp('****************************************************');
disp(' Would you like to save the filterbank coefficients ');
disp(' to a file? ');
disp(' [enter] -- no ');
disp(' filename - yes');

name=input('Choose:','s');
  if exist('cf') CentFreqs=cf; else CentFreqs=[];end
if ~isempty(name),
  eval(['save ' name '.mat f b CentFreqs']);
end



