% Profile_Dau96
%
% This profile follows the model presented in 
% (Dau, Puschel, and Kohlrausch, 1996).
% 
% 
% This file is a part of HUTear- Matlab toolbox for auditory
% modeling. The toolbox is available at 
% http://www.acoustics.hut.fi/software/HUTear/

% Copyrights: Aki Hrm, Helsinki University of Technology, 
% Laboratory of Acoustics and Audio Signal Processing, 
% Espoo, Finland.
% Date: August 20 1999
% Email: Aki.Harma@hut.fi
clear model;

% General parameters
model.id = 'An auditory model by (Dau, Puschel, and Kohlrausch, 1996';
model.fs = 44100; % Sampling frequency
model.ds = 1;     % Outputs downsampled to rate fs/ds

% Outer and middle ear filtering
model.outmid.id='Fixed filter'; 
model.outmid.file='MAF255';

% Model for inner ear 
model.cochlea.id='Gammatone filterbank';
model.cochlea.gt.design='gammatone'; 
model.cochlea.gt.nch=64;
model.cochlea.gt.frange=[200 22050];

% Mechanical to Neural 
model.haircell.id = 'Half-wave rectification and LP filtering';
% Rectification
model.haircell.rcf.r='half';
% Compression
model.haircell.rcf.c=1; % No compression (power to 1)
% Smoothing
model.haircell.rcf.f='1kHz';
% Thresholding
model.haircell.rcf.thrhld=0.0;

% Neural adaptation
model.neural.id ='Non-linear adaptation network';
model.neural.function ='Dau96';
model.neural.Dau.thr=0.05;



