% Profile_Plack98
%
% This profile corresponds to a revised temporal window model 
% presented in (Plack&Oxenham, 1998). 
% 
% This file is a part of HUTear- Matlab toolbox for auditory
% modeling. The toolbox is available at 
% http://www.acoustics.hut.fi/software/HUTear/

% Copyrights: Aki Hrm, Helsinki University of Technology, 
% Laboratory of Acoustics and Audio Signal Processing, 
% Espoo, Finland.
% Date: August 20 1999
% Email: Aki.Harma@hut.fi
clear model;

% General parameters
model.id = 'Temporal window model (Plack & Oxenham, 1998)';
model.fs = 44100; % Sampling frequency
model.ds = 1;     % Outputs downsampled to rate fs/ds

% Outer and middle ear filtering
model.outmid.id='Fixed filter'; 
model.outmid.file='MAF255';

% Model for inner ear 
model.cochlea.id='Gammatone filterbank';
model.cochlea.gt.design='gammatone'; 
model.cochlea.gt.nch=64;
model.cochlea.gt.frange=[200 22050];

% Mechanical to neural - haircell
model.haircell.id = 'Half-wave rectification, compression, and _filtering';
% Rectification
model.haircell.rcf.r='half';
% Compression
model.haircell.rcf.c='Plack98';
% Smoothing
model.haircell.rcf.f='no';
% Thresholding
model.haircell.rcf.thrhld='no';

% Neural adaptation
model.neural.id ='Sliding temporal window function';
model.neural.function ='Plack98';

% Post-processing
model.pp.id = 'Decibel scale';
model.pp.mapping='decibel';






